import React from 'react'
import ReactDOM from 'react-dom/client'
import App from './App'
import './index.css'
import VercelSpeedInsights from './components/VercelSpeedInsights'

// Add the import for our cache clearing utility
import './utils/clearCache'

// Type declaration for the React DevTools global hook
declare global {
  interface Window {
    __REACT_DEVTOOLS_GLOBAL_HOOK__?: {
      renderers?: Map<any, any>;
      supportsFiber?: boolean;
      inject?: (...args: any[]) => any;
      onScheduleFiberRoot?: (...args: any[]) => any;
      onCommitFiberRoot?: (...args: any[]) => any;
      onCommitFiberUnmount?: (...args: any[]) => any;
      injectIntoGlobalHook?: (...args: any[]) => any;
      [key: string]: any;
    };
  }
}

// Add chunk loading retry logic
const MAX_RETRY_COUNT = 3;
const RETRY_DELAY = 1000; // 1 second

// Store module loading retry counts
const retryCountMap = new Map<string, number>();

// Fix for React DevTools "Cannot read properties of undefined (reading 'forEach')" error
// This happens because the React DevTools hook is not properly initialized in some environments
const installReactDevToolsHook = () => {
  try {
    // Check if the hook is already defined
    if (window.__REACT_DEVTOOLS_GLOBAL_HOOK__) {
      console.log('React DevTools hook already exists, applying safety patches');
      
      // Create a safe copy of the original hook
      const originalHook = window.__REACT_DEVTOOLS_GLOBAL_HOOK__;
      
      // Create a safer replacement for injectIntoGlobalHook that properly handles undefined values
      const safeInjectIntoGlobalHook = (target: any) => {
        try {
          if (!originalHook || typeof originalHook.injectIntoGlobalHook !== 'function') {
            console.log('Safe hook: injectIntoGlobalHook not available, creating empty implementation');
            return;
          }
          
          // Ensure the renderers property exists before it gets used
          if (!originalHook.renderers) {
            originalHook.renderers = new Map();
          }
          
          // Call the original function safely
          originalHook.injectIntoGlobalHook(target);
        } catch (error) {
          console.error('Error in safe injectIntoGlobalHook:', error);
        }
      };
      
      // Create a wrapper for the DevTools hook that handles errors gracefully
      window.__REACT_DEVTOOLS_GLOBAL_HOOK__ = new Proxy(originalHook, {
        get: (target, prop) => {
          if (prop === 'injectIntoGlobalHook') {
            return safeInjectIntoGlobalHook;
          }
          
          // For renderers property, ensure it exists
          if (prop === 'renderers' && !target.renderers) {
            target.renderers = new Map();
          }
          
          const value = target[prop as keyof typeof target];
          
          // Return safe function wrappers for methods
          if (typeof value === 'function') {
            return function(...args: any[]) {
              try {
                return value.apply(target, args);
              } catch (error) {
                console.error(`Error in React DevTools hook method '${String(prop)}':`, error);
                return undefined;
              }
            };
          }
          
          return value;
        }
      });
      
      console.log('React DevTools hook successfully patched');
    } else {
      console.log('React DevTools hook not found, creating a safe empty implementation');
      
      // Create a safe empty implementation of the hook
      window.__REACT_DEVTOOLS_GLOBAL_HOOK__ = {
        renderers: new Map(),
        supportsFiber: true,
        inject: () => {},
        onScheduleFiberRoot: () => {},
        onCommitFiberRoot: () => {},
        onCommitFiberUnmount: () => {},
        injectIntoGlobalHook: () => {},
      };
    }
  } catch (error) {
    console.error('Error installing React DevTools hook:', error);
  }
};

// Apply the patch before the application starts
installReactDevToolsHook();

// Custom error handler for module loading errors
const handleChunkLoadError = () => {
  console.error('Chunk loading failed!');
  const timestamp = Date.now();
  window.location.href = `${window.location.pathname}?reload=${timestamp}`;
  return false;
};

window.addEventListener('error', (event) => {
  if (event.message.includes('Loading chunk') && event.message.includes('failed')) {
    console.error('Chunk loading error detected, will try reloading');
    // Try to reload immediately
    const url = new URL(window.location.href);
    url.searchParams.set('reload', Date.now().toString());
    window.location.href = url.toString();
    
    // Prevent default error handling
    event.preventDefault();
  }
});

console.log('React application successfully mounted');

// Add Vercel Speed Insights in development mode
// It won't send any data in development but will help ensure it works properly
// in production
if (import.meta.env.DEV) {
  // Call the component instead of the function
  ReactDOM.createRoot(document.getElementById('root')!).render(
    <React.StrictMode>
      <App />
      <VercelSpeedInsights />
    </React.StrictMode>,
  )
} else {
  ReactDOM.createRoot(document.getElementById('root')!).render(
    <React.StrictMode>
      <App />
      <VercelSpeedInsights />
    </React.StrictMode>,
  )
} 